using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace FlexiGen {

public class Config {


#region "Static stuff"

#endregion


#region "Encapsulation"
private Generator m_Generator;
private string m_ConfigFilePath;
private Hashtable m_Defaults;
private SourceCollection m_Sources;
private TargetCollection m_Targets;
private AddInCollection m_AddIns;
private EvaluatorCollection m_Evaluators;
private XmlDocument m_doc;
#endregion


#region "Init/Fin"
internal Config() {
}
internal void Init(Generator generator, String configFilePath){
	// save
	m_Generator = generator;
	m_ConfigFilePath = configFilePath;
	// create helpers
	m_Defaults = new Hashtable();
	m_Sources = new SourceCollection();
	m_Targets = new TargetCollection();
	m_AddIns = new AddInCollection();
	m_Evaluators = new EvaluatorCollection();
	// get the config XML
	m_doc = new XmlDocument();
	m_doc.Load(m_ConfigFilePath);
	foreach(XmlNode xn in m_doc.DocumentElement){
		if(xn is XmlElement){
			XmlElement xe = (XmlElement) xn;
			switch(xe.Name){
				case "default":
					this.AddDefault(xe);
					break;
				case "addin":
					this.AddAddIn(xe);
					break;
				case "source":
					this.AddSource(xe);
					break;
				case "target":
					this.AddTarget(xe);
					break;			
			}
		}
	}
}
private void AddDefault(XmlElement xe){
	if(xe.Attributes.Count != 1){
		throw new FlexiGenConfigException("A 'default' element must have exactly one attribute.");
	}
	XmlAttribute xa = xe.Attributes[0];
	if(m_Defaults.ContainsKey(xa.Name)){
		throw new FlexiGenConfigException("There already is a default value for '" + xa.Name + "'.");
	}
	m_Defaults.Add(xa.Name, xa.Value);
}
private void AddAddIn(XmlElement xe){
	AddIn ai = new AddIn(this.Generator, xe);
	this.AddIns.Add(ai);
}
private void AddSource(XmlElement xe){
	string sourceTypeName = this.Generator.FindAttribute(xe, "sourceType");
	ISource src = (ISource) this.Generator.CreateInstance(sourceTypeName, typeof(ISource));
	if(src is IInitializeable){
		((IInitializeable)src).Initialize(this.Generator, xe);
	}
	this.Sources.Add(src);
}
private void AddTarget(XmlElement xe){
	// ...
}
#endregion


#region "Event handlers"

#endregion


#region "Event raisers"

#endregion


#region "Properties"
public Generator Generator{
	get{
		return m_Generator;
	}
}
public SourceCollection Sources{
	get{
		return m_Sources;
	}
}
public TargetCollection Targets{
	get{
		return m_Targets;
	}
}
public AddInCollection AddIns{
	get{
		return m_AddIns;
	}
}
public EvaluatorCollection Evaluators{
	get{
		return m_Evaluators;
	}
}
public string ConfigFilePath{
	get{
		return m_ConfigFilePath;
	}
}
public XmlDocument TemplateDocument{
	get{
		return m_doc;
	}
}
#endregion


#region "Methods"
public string GetDefaultAttribute(string name){
	object obj = m_Defaults[name];
	if(obj != null){
		return obj.ToString();
	}
	return null;
}
#endregion


#region "Helpers"

#endregion


} // end class Config

} // end namespace FlexiGen
